function titr = diprotic2(CA,CB,volA,volB,pKAA1,pKAA2,pKBB,hhplot)
% Plots a titration curve for titrating a weak acid with a weak base
% Outputs a matrix with columns being:
%  volume of B,pH,[H2A],[HA-],[A2-],[B],[BH+],buffer capacity
% Jason Kahn U Maryland 4/1/2018  version 1.03 (derived from weakweak4)
% [titr = ]diprotic1(CA,CB,volA,volB,pKAA1,pKAA2,pKBB,hhplot);  for example:
% titr = diprotic1(.1,.05,100,500,2.75,6.31,3.5,0); % or just
% diprotic1(.75,1.0,200,450,3.10,6.15,-5,0); % for titration with a strong base
% if hhplot is set to 1, will also plot the pH derived from the
% Henderson-Hasselbalch just assuming complete neutralization to give [HA-]
% and [A2-] in each segment.
% This was all independently programmed, but it is similar to
%  http://www.chembuddy.com/?left=BATE&right=pH-calculator
% Titration of a weak acid A with concentration CA and volume volA with a
% weak base B of concentration (CB), from a volume of zero up to volB.
% KAA1 = 10^(-pKAA1) is the first acid dissociation constant of A,
% KAA2 = 10^(-pKAA2) is the second acid dissociation constant of A, which must
% be greater than KAA1, and KBB = 10^(-pKBB) is the base dissociation constant of B
% Volume units do not matter (as long as they are the same!), typically they would be ml.
% Returns a matrix with columns volB, pH,[H2A],[HA],[A-],[B],[BH+],bufcap in the "titr" variable.
% The program graphs the first two versus each other to show a typical titration.
% Save the values returned in titr if you need to do multiple plots with different x
% scales or look at speciation. Otherwise the default is to overlay plots, so to do separate
% plots with different axes, give the "figure" command between calls to
% diprotic1.
% For example, to overlay curves showing the effect of dilution
% titr1 = diprotic1(.1,.03,100,800,2.75,5.22,3.5,1);
% titr2 = diprotic1(.01,.003,100,800,2.75,5.22,3.5,1);
% titr3 = diprotic1(.001,.0003,100,800,2.75,5.22,3.5,1);
% titr4 = diprotic1(.0001,.00003,100,800,2.75,5.22,3.5,1);
% To plot the distribution of H2A,HA-, A2- species use these commands:
% >> titr2 = diprotic1(.1,.05,100,500,2.75,6.31,3.5,0);
% >> figure  % So you don't plot on top of the pH plot
% >> plot(titr2(:,1),titr2(:,3),titr2(:,1),titr2(:,4),titr2(:,1),titr2(:,5))
% >> legend('H2A','HA-','A2-')
% (and similarly can plot B and BH+)
% To plot the distribution of H2A,HA-, A2- species as a function of pH:
% >> titr2 = diprotic1(.1,.05,100,500,2.75,6.31,3.5,0);
% >> figure  % So you don't plot on top of the pH plot
% >> plot(titr2(:,2),titr2(:,3),titr2(:,2),titr2(:,4),titr2(:,2),titr2(:,5))
% >> legend('H2A','HA-','A2-')
% (and similarly can plot B and BH+)
% To plot the above distribution and buffer capacity as a double-y:
% figure
% yyaxis left
% plot(titr2(:,2),titr2(:,3),titr2(:,2),titr2(:,4),titr2(:,2),titr2(:,5))
% yyaxis right
% plot(titr2(:,2),titr2(:,8))
% legend('H2A','HA-','A2-','Bufcap')
KW = 1e-14;
KAA1 = 10^(-pKAA1);
KAA2 = 10^(-pKAA2);
if (KAA1 < KAA2)
    disp('Please enter pKA1 < pKa2. Exiting.')
    return
end
KBB = 10^(-pKBB);
KWoKB = KW/KBB;
%hhplot = 0; Should install this as a varargin etc.
npts = 301;  % This is a minimum number, probably must be > 200
% Figure out where the equivalence points are and make points denser in the vicinity. 
eqpt = 2*volA*CA/CB;
disp(['Equiv pt at ',num2str(eqpt)])
if volB > 1.03*eqpt   
    Bvol = [ 0 : (volB-.06*eqpt)/(npts-101) : 0.95*eqpt/2-.06*eqpt/100 , ...
        0.95*eqpt/2 : .03*eqpt/100 : 0.97*eqpt/2-0.03*eqpt/100 ,...
        0.97*eqpt/2 : .01*eqpt/100 : 1.03*eqpt/2-0.01*eqpt/100 ,...
        1.03*eqpt/2 : .03*eqpt/100 : 1.06*eqpt/2-0.03*eqpt/100 ,...
        1.06*eqpt/2 : (volB-.06*eqpt)/(npts-101) : 0.97*eqpt-.06*eqpt/100 ,...
        0.97*eqpt : .06*eqpt/100 : eqpt-.06*eqpt/100 ,...
        eqpt-.06*eqpt/100+eqpt*2e-5 : eqpt*2e-5 : eqpt+.06*eqpt/100-eqpt*2e-5,...
        eqpt+.06*eqpt/100:.06*eqpt/100:1.03*eqpt ,...
        1.03*eqpt+.06*eqpt/100 : (volB-.06*eqpt)/(npts-101) : volB ];
    if Bvol(end) ~= volB
        Bvol = [Bvol volB];  % For the axes to plot neatly want the fixed end point
    end
    npts = length(Bvol);
    disp(['New npts ', num2str(npts)])
else
    Bvol = 0 : volB/(npts-1) : volB;
end
Bconcs = CB*Bvol./(volA+Bvol);
Aconcs = CA*volA./(volA+Bvol);
% HH doesn't work yet for diprotic
if (hhplot == 1)  % Plot the result of just plugging in to HH equation
    HH1 = find(Bvol < eqpt/2,1,'last');
    HH2 = find(Bvol < eqpt,1,'last');
    Hhph1 = pKAA1 + log10(Bconcs(1:HH1)./(Aconcs(1:HH1)-Bconcs(1:HH1)));
    Hhph2 = pKAA2 + log10( CB*(Bvol((HH1+2):HH2)-eqpt/2)./(volA+Bvol((HH1+2):HH2)) ...
        ./(CA*volA./(volA+Bvol((HH1+2):HH2)) - CB*(Bvol((HH1+2):HH2)-eqpt/2)./(volA+Bvol((HH1+2):HH2))));
    Hhph = [Hhph1 0.5*(pKAA1+pKAA2) Hhph2];
end
proton = zeros(npts,1);
H2A = zeros(npts,1);
HAminus = zeros(npts,1);
Aminus2 = zeros(npts,1);
BaseB = zeros(npts,1);
BHplus =  zeros(npts,1);
pH = zeros(npts,1);
Bufcap = zeros(npts,1);
for i=1:npts
    a1 = 1;
    a2 = Bconcs(i) + KAA1 + KWoKB;
    a3 = KAA1*(KAA2 + Bconcs(i) - Aconcs(i)) + KAA1*KWoKB - KW;
    a4 = -Aconcs(i)*KAA1*KWoKB - KW*(KAA1 + KWoKB) + KAA1*KAA2*(KWoKB + Bconcs(i) - 2*Aconcs(i));
    a5 = -KW*KAA1*KWoKB -KAA1*KAA2*(KW + 2*Aconcs(i)*KWoKB);
    a6 = -KAA1*KAA2*KW*KWoKB;
    poly = [a1 a2 a3 a4 a5 a6];
    rpoly = roots(poly);   % This is where all the action happens
    proton(i) = rpoly(abs(rpoly > 0) & imag(rpoly) == 0); % Choose the positive real root
    pH(i) = -log10(proton(i));
    HAminus(i) = Aconcs(i)./(1 + proton(i)./KAA1 + KAA2./proton(i));
    H2A(i)  = proton(i).*HAminus(i)/KAA1; 
    Aminus2(i) = Aconcs(i)-HAminus(i)-H2A(i);
    BaseB(i) = Bconcs(i)*(KWoKB/proton(i))/(1+KWoKB/proton(i));
    BHplus(i) = Bconcs(i)-BaseB(i);
    if i > 2
        Bufcap(i-1) = 0.5*((Bconcs(i)-Bconcs(i-1))/(pH(i)-pH(i-1)) ...
            + ((Bconcs(i-1)-Bconcs(i-2))/(pH(i-1)-pH(i-2))));
    end
end
% The buffer capacity calculation is dubious for a weak base titrant.
Bufcap(1) = (Bconcs(2)-Bconcs(1))/(pH(2)-pH(1));
Bufcap(npts) = (Bconcs(npts)-Bconcs(npts-1))/(pH(npts)-pH(npts-1));
titr = [Bvol' pH H2A HAminus Aminus2 BaseB BHplus Bufcap];  % Returns a matrix with columns as shown
% The rest is plotting the pH vs. the added volume
figure  % Uncomment this line to prevent overlaying plots, which should not be
%the default or diprotic because it makes two plots. Should have an optional argument to control this from the
%command line.
subplot(3,1,1)
box on
hold on
plot(titr(:,1),titr(:,2),'LineWidth',2);
plot(titr(:,1),titr(:,2),'o');   % to see points
if (hhplot ==1)
    plot(Bvol(1:HH2),Hhph,'b-')
end
axis([0 max(Bvol) 0 14])    % So different titrations can be compared
xlimit = xlim;
numxticks = max(4*Bvol/eqpt); % should make this adaptive somehow.
set(gca,'FontSize',14,'XTick',xlimit(1) : (xlimit(2)- xlimit(1))/numxticks...
    : xlimit(2))
title('Acid-Base Titration','FontWeight','bold','FontSize',16)
xlabel('Added volume of Base','FontWeight','bold','FontSize',14);
ylabel('pH','FontWeight','bold','FontSize',14);
text(0.10*max(Bvol),0.75,...
     ['pKa1(A) = ',num2str(-log10(KAA1),'%4.1f'),...
     '  pKa2(A) = ',num2str(-log10(KAA2),'%4.1f'),...
     '  pKb(B) = ',num2str(-log10(KBB),'%4.1f'), ...
     '  CA = ',num2str(CA),' M ',...
     '  CB = ',num2str(CB),' M ',...
     '  Init. vol of A = ',num2str(volA)],...
     'Fontsize',14,'FontWeight','bold')
% uncomment line below to save (should rewrite to use export_fig).
%print -dpng -r200
%figure
subplot(3,1,2)
yyaxis left
plot(titr(:,1),titr(:,3),titr(:,1),titr(:,4),titr(:,1),titr(:,5),'LineWidth',2)
xlabel('Added Volume of Base','FontWeight','bold','FontSize',14);
ylabel('Concentrations','FontWeight','bold','FontSize',14);
title('Speciation vs. volume','FontWeight','bold','FontSize',16)
yyaxis right
plot(titr(:,1),titr(:,8))
ylabel('Buffer Capacity','FontWeight','bold','FontSize',14);
legend('H2A','HA-','A2-','Bufcap','Location','east')
text(0.1*volB,0.90*max(Bufcap),...
     ['pKa1(A) = ',num2str(-log10(KAA1),'%4.1f'),...
     '  pKa2(A) = ',num2str(-log10(KAA2),'%4.1f'),...
     '  pKb(B) = ',num2str(-log10(KBB),'%4.1f')],...
     'Fontsize',14);
set(gca,'FontSize',14)
subplot(3,1,3)
yyaxis left
plot(titr(:,2),titr(:,3),titr(:,2),titr(:,4),titr(:,2),titr(:,5),'LineWidth',2)
xlabel('pH','FontWeight','bold','FontSize',14);
ylabel('Concentrations)','FontWeight','bold','FontSize',14);
title('Speciation vs. pH','FontWeight','bold','FontSize',16)
yyaxis right
plot(titr(:,2),titr(:,8))
ylabel('Buffer Capacity','FontWeight','bold','FontSize',14);
legend('H2A','HA-','A2-','Bufcap','Location','east')
text(4,0.90*max(Bufcap),...
     ['pKa1(A) = ',num2str(-log10(KAA1),'%4.1f'),...
     '  pKa2(A) = ',num2str(-log10(KAA2),'%4.1f'),...
     '  pKb(B) = ',num2str(-log10(KBB),'%4.1f')],...
     'Fontsize',14);
set(gca,'FontSize',14)
return
